/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.modules.bigbluebutton.manager;

import java.util.Date;
import java.util.List;
import java.util.UUID;

import org.junit.Assert;
import org.junit.Test;
import org.olat.core.commons.persistence.DB;
import org.olat.core.id.Identity;
import org.olat.group.BusinessGroup;
import org.olat.group.manager.BusinessGroupDAO;
import org.olat.modules.bigbluebutton.BigBlueButtonMeeting;
import org.olat.modules.bigbluebutton.BigBlueButtonRecording;
import org.olat.modules.bigbluebutton.BigBlueButtonRecordingReference;
import org.olat.modules.bigbluebutton.model.BigBlueButtonRecordingImpl;
import org.olat.test.JunitTestHelper;
import org.olat.test.OlatTestCase;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 
 * Initial date: 7 août 2020<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
public class BigBlueButtonRecordingReferenceDAOTest extends OlatTestCase {
	
	@Autowired
	private DB dbInstance;
	@Autowired
	private BusinessGroupDAO businessGroupDao;
	@Autowired
	private BigBlueButtonMeetingDAO bigBlueButtonMeetingDao;
	@Autowired
	private BigBlueButtonRecordingReferenceDAO bigBlueButtonRecordingReferenceDao;
	
	@Test
	public void createReference() {
		Identity id = JunitTestHelper.createAndPersistIdentityAsRndUser("bbb-record-1");
		BusinessGroup group = businessGroupDao.createAndPersist(null, "BBB Recording 1", "bbb-desc", -1, -1, false, false, false, false, false);
		BigBlueButtonMeeting meeting = bigBlueButtonMeetingDao.createAndPersistMeeting("Record all - 1", null, null, group, id);
		dbInstance.commit();
		
		BigBlueButtonRecording recording = BigBlueButtonRecordingImpl.valueOf(UUID.randomUUID().toString(), "Recorded", UUID.randomUUID().toString(),
				new Date(), new Date(), "http://button.openolat.com", "presentation");
		BigBlueButtonRecordingReference reference = bigBlueButtonRecordingReferenceDao.createReference(recording, meeting, null);
		dbInstance.commitAndCloseSession();
		
		Assert.assertNotNull(reference);
		Assert.assertNotNull(reference.getCreationDate());
		Assert.assertNotNull(reference.getLastModified());
		Assert.assertNotNull(reference.getStartDate());
		Assert.assertNotNull(reference.getEndDate());
		Assert.assertEquals(meeting, reference.getMeeting());
		Assert.assertEquals("presentation", reference.getType());
		Assert.assertEquals("http://button.openolat.com", reference.getUrl());
	}
	
	@Test
	public void getRecordingReferences() {
		Identity id = JunitTestHelper.createAndPersistIdentityAsRndUser("bbb-record-2");
		BusinessGroup group = businessGroupDao.createAndPersist(null, "BBB Recording 2", "bbb-desc", -1, -1, false, false, false, false, false);
		BigBlueButtonMeeting meeting = bigBlueButtonMeetingDao.createAndPersistMeeting("Recording - 2", null, null, group, id);

		BigBlueButtonRecording recording = BigBlueButtonRecordingImpl.valueOf(UUID.randomUUID().toString(), "Recorded always", UUID.randomUUID().toString(),
				new Date(), new Date(), "http://button.openolat.com/2", "presentation");
		BigBlueButtonRecordingReference reference = bigBlueButtonRecordingReferenceDao.createReference(recording, meeting, null);
		dbInstance.commitAndCloseSession();
		
		List<BigBlueButtonRecordingReference> references = bigBlueButtonRecordingReferenceDao.getRecordingReferences(meeting);
		Assert.assertNotNull(references);
		Assert.assertEquals(1, references.size());
		Assert.assertEquals(reference, references.get(0));
		
		BigBlueButtonRecordingReference reloadReference = references.get(0);
		Assert.assertNotNull(reloadReference);
		Assert.assertNotNull(reloadReference.getCreationDate());
		Assert.assertNotNull(reloadReference.getLastModified());
		Assert.assertNotNull(reloadReference.getStartDate());
		Assert.assertNotNull(reloadReference.getEndDate());
		Assert.assertEquals(meeting, reloadReference.getMeeting());
		Assert.assertEquals("presentation", reloadReference.getType());
		Assert.assertEquals("http://button.openolat.com/2", reloadReference.getUrl());
	}
}
